package com.sun.java.swing.plaf.windows;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.lang.ref.WeakReference;
import javax.swing.plaf.ColorUIResource;
import javax.swing.plaf.FontUIResource;
import java.awt.Font;
import java.awt.Color;
import javax.swing.LookAndFeel;
import java.awt.Toolkit;
import javax.swing.SwingUtilities;
import java.awt.Window;
import java.awt.Frame;
import javax.swing.UIManager;
import java.lang.ref.ReferenceQueue;
import javax.swing.UIDefaults;

public class DesktopProperty implements UIDefaults.ActiveValue
{
    private static boolean updatePending;
    private static final ReferenceQueue<DesktopProperty> queue;
    private DesktopProperty.WeakPCL pcl;
    private final String key;
    private Object value;
    private final Object fallback;
    
    static void flushUnreferencedProperties() {
        DesktopProperty.WeakPCL weakPCL;
        while ((weakPCL = (DesktopProperty.WeakPCL)DesktopProperty.queue.poll()) != null) {
            weakPCL.dispose();
        }
    }
    
    private static synchronized void setUpdatePending(boolean updatePending) {
        DesktopProperty.updatePending = updatePending;
    }
    
    private static synchronized boolean isUpdatePending() {
        return DesktopProperty.updatePending;
    }
    
    private static void updateAllUIs() {
        if (UIManager.getLookAndFeel().getClass().getPackage().equals(DesktopProperty.class.getPackage())) {
            XPStyle.invalidateStyle();
        }
        Frame[] frames = Frame.getFrames();
        for (int length = frames.length, i = 0; i < length; ++i) {
            updateWindowUI(frames[i]);
        }
    }
    
    private static void updateWindowUI(Window c) {
        SwingUtilities.updateComponentTreeUI(c);
        Window[] ownedWindows = c.getOwnedWindows();
        for (int length = ownedWindows.length, i = 0; i < length; ++i) {
            updateWindowUI(ownedWindows[i]);
        }
    }
    
    public DesktopProperty(String key, Object fallback) {
        this.key = key;
        this.fallback = fallback;
        flushUnreferencedProperties();
    }
    
    @Override
    public Object createValue(UIDefaults uiDefaults) {
        if (this.value == null) {
            this.value = this.configureValue(this.getValueFromDesktop());
            if (this.value == null) {
                this.value = this.configureValue(this.getDefaultValue());
            }
        }
        return this.value;
    }
    
    protected Object getValueFromDesktop() {
        Toolkit defaultToolkit = Toolkit.getDefaultToolkit();
        if (this.pcl == null) {
            this.pcl = new DesktopProperty.WeakPCL(this, this.getKey(), UIManager.getLookAndFeel());
            defaultToolkit.addPropertyChangeListener(this.getKey(), this.pcl);
        }
        return defaultToolkit.getDesktopProperty(this.getKey());
    }
    
    protected Object getDefaultValue() {
        return this.fallback;
    }
    
    public void invalidate(LookAndFeel lookAndFeel) {
        this.invalidate();
    }
    
    public void invalidate() {
        this.value = null;
    }
    
    protected void updateUI() {
        if (!isUpdatePending()) {
            setUpdatePending(true);
            SwingUtilities.invokeLater(new Runnable() {
                @Override
                public void run() {
                    DesktopProperty.access$000();
                    DesktopProperty.access$100(false);
                }
            });
        }
    }
    
    protected Object configureValue(Object o) {
        if (o != null) {
            if ((o instanceof Color)) {
                return new ColorUIResource((Color)o);
            }
            if ((o instanceof Font)) {
                return new FontUIResource((Font)o);
            }
            if ((o instanceof UIDefaults.LazyValue)) {
                o = ((UIDefaults.LazyValue)o).createValue(null);
            }
            else if ((o instanceof UIDefaults.ActiveValue)) {
                o = ((UIDefaults.ActiveValue)o).createValue(null);
            }
        }
        return o;
    }
    
    protected String getKey() {
        return this.key;
    }
    
    static void access$000() {
        updateAllUIs();
    }
    
    static void access$100(boolean updatePending) {
        setUpdatePending(updatePending);
    }
    
    static {
        queue = new ReferenceQueue();
    }
    
    private static class WeakPCL extends WeakReference<DesktopProperty> implements PropertyChangeListener
    {
        private String key;
        private LookAndFeel laf;
        
        WeakPCL(DesktopProperty referent, String key, LookAndFeel laf) {
            super(referent, DesktopProperty.queue);
            this.key = key;
            this.laf = laf;
        }
        
        @Override
        public void propertyChange(PropertyChangeEvent propertyChangeEvent) {
            DesktopProperty desktopProperty = (DesktopProperty)this.get();
            if (desktopProperty == null || this.laf != UIManager.getLookAndFeel()) {
                this.dispose();
            }
            else {
                desktopProperty.invalidate(this.laf);
                desktopProperty.updateUI();
            }
        }
        
        void dispose() {
            Toolkit.getDefaultToolkit().removePropertyChangeListener(this.key, this);
        }
    }
}
